"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compareGradients = void 0;
exports.computePickerPosition = computePickerPosition;
exports.computeSquareXY = computeSquareXY;
exports.getGradientType = exports.getDegrees = void 0;
exports.getHandleValue = getHandleValue;
exports.safeBounds = exports.objectToString = exports.isUpperCase = exports.getNewHsl = void 0;
var _formatters = require("./formatters");
var _constants = require("../constants");
var tc = require("tinycolor2");
var barSize = _constants.config.barSize,
  crossSize = _constants.config.crossSize;
function getHandleValue(e) {
  var _safeBounds = safeBounds(e),
    offsetLeft = _safeBounds.offsetLeft,
    clientWidth = _safeBounds.clientWidth;
  var pos = e.clientX - offsetLeft - barSize / 2;
  var adjuster = clientWidth - 18;
  var bounded = (0, _formatters.formatInputValues)(pos, 0, adjuster);
  return Math.round(bounded / (adjuster / 100));
}
function computeSquareXY(hsl, squareSize, squareHeight) {
  var s = hsl[1] * 100;
  var l = hsl[2] * 100;
  var t = s * (l < 50 ? l : 100 - l) / 100;
  var s1 = Math.round(200 * t / (l + t)) | 0;
  var b1 = Math.round(t + l);
  var x = squareSize / 100 * s1 - crossSize / 2;
  var y = squareHeight - squareHeight / 100 * b1 - crossSize / 2;
  return [x, y];
}
function computePickerPosition(e) {
  var _safeBounds2 = safeBounds(e),
    offsetLeft = _safeBounds2.offsetLeft,
    offsetTop = _safeBounds2.offsetTop,
    clientWidth = _safeBounds2.clientWidth,
    clientHeight = _safeBounds2.clientHeight;
  var getX = function getX() {
    var xPos = e.clientX - offsetLeft - crossSize / 2;
    return (0, _formatters.formatInputValues)(xPos, -9, clientWidth - 10);
  };
  var getY = function getY() {
    var yPos = e.clientY - offsetTop - crossSize / 2;
    return (0, _formatters.formatInputValues)(yPos, -9, clientHeight - 10);
  };
  return [getX(), getY()];
}
var getGradientType = function getGradientType(value) {
  return value === null || value === void 0 ? void 0 : value.split("(")[0];
};
exports.getGradientType = getGradientType;
var getNewHsl = function getNewHsl(newHue, s, l, o, setInternalHue) {
  setInternalHue(newHue);
  var tiny = tc({
    h: newHue,
    s: s,
    l: l
  });
  var _tiny$toRgb = tiny.toRgb(),
    r = _tiny$toRgb.r,
    g = _tiny$toRgb.g,
    b = _tiny$toRgb.b;
  return "rgba(".concat(r, ", ").concat(g, ", ").concat(b, ", ").concat(o, ")");
};
exports.getNewHsl = getNewHsl;
var safeBounds = function safeBounds(e) {
  var client = e.target.parentNode.getBoundingClientRect();
  var className = e.target.className;
  var adjuster = className === "c-resize ps-rl" ? 15 : 0;
  return {
    offsetLeft: (client === null || client === void 0 ? void 0 : client.x) + adjuster,
    offsetTop: client === null || client === void 0 ? void 0 : client.y,
    clientWidth: client === null || client === void 0 ? void 0 : client.width,
    clientHeight: client === null || client === void 0 ? void 0 : client.height
  };
};
exports.safeBounds = safeBounds;
var isUpperCase = function isUpperCase(str) {
  var _str$;
  return (str === null || str === void 0 ? void 0 : str[0]) === (str === null || str === void 0 ? void 0 : (_str$ = str[0]) === null || _str$ === void 0 ? void 0 : _str$.toUpperCase());
};
exports.isUpperCase = isUpperCase;
var compareGradients = function compareGradients(g1, g2) {
  var _g1$toLowerCase, _g2$toLowerCase;
  var ng1 = g1 === null || g1 === void 0 ? void 0 : (_g1$toLowerCase = g1.toLowerCase()) === null || _g1$toLowerCase === void 0 ? void 0 : _g1$toLowerCase.replaceAll(" ", "");
  var ng2 = g2 === null || g2 === void 0 ? void 0 : (_g2$toLowerCase = g2.toLowerCase()) === null || _g2$toLowerCase === void 0 ? void 0 : _g2$toLowerCase.replaceAll(" ", "");
  if (ng1 === ng2) {
    return true;
  } else {
    return false;
  }
};
exports.compareGradients = compareGradients;
var convertShortHandDeg = function convertShortHandDeg(dir) {
  if (dir === "to top") {
    return 0;
  } else if (dir === "to bottom") {
    return 180;
  } else if (dir === "to left") {
    return 270;
  } else if (dir === "to right") {
    return 90;
  } else if (dir === "to top right") {
    return 45;
  } else if (dir === "to bottom right") {
    return 135;
  } else if (dir === "to bottom left") {
    return 225;
  } else if (dir === "to top left") {
    return 315;
  } else {
    var safeDir = dir || 0;
    return parseInt(safeDir);
  }
};
var getDegrees = function getDegrees(value) {
  var _s1$split$;
  var s1 = value === null || value === void 0 ? void 0 : value.split(",")[0];
  var s2 = s1 === null || s1 === void 0 ? void 0 : (_s1$split$ = s1.split("(")[1]) === null || _s1$split$ === void 0 ? void 0 : _s1$split$.replace("deg", "");
  return convertShortHandDeg(s2);
};
exports.getDegrees = getDegrees;
var objectToString = function objectToString(value) {
  if (typeof value === 'string') {
    return value;
  } else {
    var _value$type;
    if (value !== null && value !== void 0 && (_value$type = value.type) !== null && _value$type !== void 0 && _value$type.includes("gradient")) {
      var _value$colorStops, _sorted$map, _value$orientation;
      var sorted = value === null || value === void 0 ? void 0 : (_value$colorStops = value.colorStops) === null || _value$colorStops === void 0 ? void 0 : _value$colorStops.sort(function (a, b) {
        return (a === null || a === void 0 ? void 0 : a.left) - (b === null || b === void 0 ? void 0 : b.left);
      });
      var string = sorted === null || sorted === void 0 ? void 0 : (_sorted$map = sorted.map(function (c) {
        return "".concat(c === null || c === void 0 ? void 0 : c.value, " ").concat(c === null || c === void 0 ? void 0 : c.left, "%");
      })) === null || _sorted$map === void 0 ? void 0 : _sorted$map.join(", ");
      var type = value === null || value === void 0 ? void 0 : value.type;
      var degs = convertShortHandDeg(value === null || value === void 0 ? void 0 : (_value$orientation = value.orientation) === null || _value$orientation === void 0 ? void 0 : _value$orientation.value);
      var gradientStr = type === 'linear-gradient' ? "".concat(degs, "deg") : 'circle';
      return "".concat(type, "(").concat(gradientStr, ", ").concat(string, ")");
    } else {
      var _value$colorStops$;
      var color = (value === null || value === void 0 ? void 0 : (_value$colorStops$ = value.colorStops[0]) === null || _value$colorStops$ === void 0 ? void 0 : _value$colorStops$.value) || 'rgba(175, 51, 242, 1)';
      return color;
    }
  }
};
exports.objectToString = objectToString;